import { useState, useEffect } from "react";
import { Settings } from "../../../lib/settings";
import { useRefresh } from "../../../lib/RefreshContext";

interface InviteCodeUse {
  usedBy: string;
  usedAt: string;
}

interface InviteCode {
  code: string;
  available: number;
  disabled: boolean;
  forAccount: string;
  createdBy: string;
  createdAt: string;
  uses: InviteCodeUse[];
}

async function getAuthHeader(): Promise<string> {
  const adminPassword = await Settings.getAdminPassword();
  const authString = `admin:${adminPassword}`;
  const base64Auth = btoa(authString);
  return `Basic ${base64Auth}`;
}

export default function InviteCodes() {
  const [codes, setCodes] = useState<InviteCode[]>([]);
  const [loading, setLoading] = useState(false);
  const [error, setError] = useState<string | null>(null);
  const [showDisabled, setShowDisabled] = useState(false);

  const { lastUpdate } = useRefresh();

  const resetState = () => {
    setCodes([]);
    setError(null);
    setLoading(false);
  };

  const filterAndSortCodes = (codes: InviteCode[]) => {
    return [...codes]
      .filter((code) => showDisabled || !code.disabled)
      .sort((a, b) => {
        if (a.disabled && !b.disabled) return 1;
        if (!a.disabled && b.disabled) return -1;
        return new Date(b.createdAt).getTime() - new Date(a.createdAt).getTime();
      });
  };

  const fetchCodes = async () => {
    try {
      setLoading(true);
      setError(null);

      const serviceUrl = await Settings.getServiceUrl();
      const authHeader = await getAuthHeader();

      const url = `${serviceUrl}/xrpc/com.atproto.admin.getInviteCodes`;
      const headers = {
        Authorization: authHeader,
        "Content-Type": "application/json",
      };

      const params = new URLSearchParams({
        sort: "recent",
        limit: "100",
      });

      const response = await fetch(`${url}?${params}`, { headers });

      if (!response.ok) {
        const errorText = await response.text();
        console.error("Error response:", errorText);
        resetState();
        throw new Error(
          `Failed to fetch codes: ${response.status} ${response.statusText}`,
        );
      }

      const data = await response.json();
      setCodes(data.codes || []);
    } catch (err) {
      console.error("Fetch error:", err);
      resetState();
      setError(err instanceof Error ? err.message : "Failed to fetch codes");
    } finally {
      setLoading(false);
    }
  };

  const createCode = async () => {
    try {
      setLoading(true);
      setError(null);

      const serviceUrl = await Settings.getServiceUrl();
      const authHeader = await getAuthHeader();

      const url = `${serviceUrl}/xrpc/com.atproto.server.createInviteCode`;
      const headers = {
        Authorization: authHeader,
        "Content-Type": "application/json",
      };

      const response = await fetch(url, {
        method: "POST",
        headers,
        body: JSON.stringify({
          useCount: 1,
        }),
      });

      if (!response.ok) {
        const errorText = await response.text();
        console.error("Error response:", errorText);
        throw new Error(
          `Failed to create code: ${response.status} ${response.statusText}`,
        );
      }

      await fetchCodes();
    } catch (err) {
      console.error("Create error:", err);
      setError(err instanceof Error ? err.message : "Failed to create code");
    } finally {
      setLoading(false);
    }
  };

  const disableCode = async (code: string) => {
    try {
      setLoading(true);
      setError(null);

      const serviceUrl = await Settings.getServiceUrl();
      const authHeader = await getAuthHeader();

      const url = `${serviceUrl}/xrpc/com.atproto.admin.disableInviteCodes`;
      const headers = {
        Authorization: authHeader,
        "Content-Type": "application/json",
      };

      const response = await fetch(url, {
        method: "POST",
        headers,
        body: JSON.stringify({
          codes: [code],
        }),
      });

      if (!response.ok) {
        const errorText = await response.text();
        console.error("Error response:", errorText);
        throw new Error(
          `Failed to disable code: ${response.status} ${response.statusText}`,
        );
      }

      await fetchCodes();
    } catch (err) {
      console.error("Disable error:", err);
      setError(err instanceof Error ? err.message : "Failed to disable code");
    } finally {
      setLoading(false);
    }
  };

  useEffect(() => {
    return () => {
      resetState();
    };
  }, []);

  useEffect(() => {
    if (typeof window !== "undefined") {
      fetchCodes();
    }
  }, [lastUpdate]);

  if (typeof window === "undefined") {
    return <div></div>;
  }

  return (
    <div className="card bg-base-100 shadow-xl">
      <div className="card-body">
        <div className="flex justify-between items-center mb-4">
          <div className="flex items-center gap-4">
            <h1 className="card-title text-2xl">Invite Codes</h1>
            <label className="cursor-pointer label gap-2">
              <span className="label-text">Show Disabled Codes</span>
              <input
                type="checkbox"
                className="toggle toggle-sm toggle-primary"
                checked={showDisabled}
                onChange={(e) => setShowDisabled(e.target.checked)}
              />
            </label>
          </div>
          <button
            onClick={createCode}
            disabled={loading}
            className="btn btn-primary"
          >
            {loading ? (
              <span className="loading loading-spinner loading-sm"></span>
            ) : (
              "Create New Code"
            )}
          </button>
        </div>

        {error && (
          <div className="alert alert-error">
            <svg
              xmlns="http://www.w3.org/2000/svg"
              className="stroke-current shrink-0 h-6 w-6"
              fill="none"
              viewBox="0 0 24 24"
            >
              <path
                strokeLinecap="round"
                strokeLinejoin="round"
                strokeWidth="2"
                d="M10 14l2-2m0 0l2-2m-2 2l-2-2m2 2l2 2m7-2a9 9 0 11-18 0 9 9 0 0118 0z"
              />
            </svg>
            <div>
              <h3 className="font-bold">Error!</h3>
              <div className="text-xs">{error}</div>
            </div>
            <button onClick={fetchCodes} className="btn btn-sm">
              Retry
            </button>
          </div>
        )}

        {loading ? (
          <div className="flex justify-center p-8">
            <span className="loading loading-spinner loading-lg"></span>
          </div>
        ) : (
          <div className="space-y-4">
            {filterAndSortCodes(codes).map((code, index) => (
              <div
                key={index}
                className={`card ${code.disabled ? "bg-base-300 opacity-75" : "bg-base-200"
                  } hover:shadow-md transition-shadow`}
              >
                <div className="card-body">
                  <div className="flex justify-between items-start">
                    <div
                      className={`font-mono text-lg bg-base-100 p-3 rounded-box select-all flex-1 mr-4 ${code.disabled ? "opacity-50" : ""
                        }`}
                    >
                      {code.code}
                    </div>
                    <div className="flex flex-col items-end gap-2">
                      {code.disabled ? (
                        <span className="badge badge-error">Disabled</span>
                      ) : (
                        <button
                          onClick={() => disableCode(code.code)}
                          className="btn btn-sm btn-warning"
                          title="Disable code"
                        >
                          Disable
                        </button>
                      )}
                    </div>
                  </div>
                  <div className="flex flex-wrap gap-4 mt-2 text-sm">
                    {!code.disabled && (
                      <div className="badge badge-neutral">
                        Uses: {code.uses.length} / {code.available}
                      </div>
                    )}
                    <div className="badge badge-ghost">
                      Created: {new Date(code.createdAt).toLocaleString()}
                    </div>
                    {code.uses.length > 0 && (
                      <div className="badge badge-info">
                        {code.uses.length}{" "}
                        {code.uses.length === 1 ? "use" : "uses"}
                      </div>
                    )}
                  </div>
                  {code.uses.length > 0 && (
                    <div
                      className={`text-sm mt-2 ${code.disabled ? "opacity-75" : "opacity-70"
                        }`}
                    >
                      <span className="font-semibold">Used by:</span>{" "}
                      {code.uses.map((use) => use.usedBy).join(", ")}
                    </div>
                  )}
                </div>
              </div>
            ))}
            {!loading && codes.length === 0 && (
              <div className="text-center p-8 bg-base-200 rounded-box">
                <div className="text-base-content/70">
                  No invite codes found
                </div>
              </div>
            )}
          </div>
        )}
      </div>
    </div>
  );
}
