import { useState, useEffect } from "react";
import { Settings } from "../../../lib/settings";
import { useRefresh } from "../../../lib/RefreshContext";

interface User {
  did: string;
  handle?: string;
  email?: string;
  displayName?: string;
  description?: string;
  avatar?: {
    ref: {
      $link: string;
    };
  };
  createdAt: string;
}


interface RepoResponse {
  cursor: string;
  repos: {
    did: string;
    head: string;
    rev: string;
    active: boolean;
  }[];
}

async function getAuthHeader(): Promise<string> {
  const adminPassword = await Settings.getAdminPassword();
  const authString = `admin:${adminPassword}`;
  const base64Auth = btoa(authString);
  return `Basic ${base64Auth}`;
}

function UserAvatar({ user }: { user: User }) {
  const [imageUrl, setImageUrl] = useState<string | null>(null);

  useEffect(() => {
    async function loadImageUrl() {
      if (user.avatar) {
        const serviceUrl = await Settings.getServiceUrl();
        setImageUrl(
          `${serviceUrl}/xrpc/com.atproto.sync.getBlob?did=${user.did}&cid=${user.avatar.ref.$link}`,
        );
      }
    }
    loadImageUrl();
  }, [user]);

  if (!user.avatar || !imageUrl) {
    return (
      <div className="avatar placeholder">
        <div className="bg-neutral-focus text-neutral-content rounded-full w-16">
          <span className="text-xl">{user.displayName?.[0] || "?"}</span>
        </div>
      </div>
    );
  }

  return (
    <img
      src={imageUrl}
      alt={user.displayName || "User avatar"}
      className="w-16 h-16 rounded-full object-cover"
    />
  );
}

export default function UserList() {
  const [users, setUsers] = useState<User[]>([]);
  const [loading, setLoading] = useState(false);
  const [error, setError] = useState<string | null>(null);
  const [settingsLoaded, setSettingsLoaded] = useState(false);
  const [hasValidSettings, setHasValidSettings] = useState(false);
  const { lastUpdate } = useRefresh();
  const [successMessage, setSuccessMessage] = useState<string | null>(null);
  const [deleteConfirmation, setDeleteConfirmation] = useState<{
    show: boolean;
    user: User | null;
  }>({
    show: false,
    user: null,
  });

  const [editModal, setEditModal] = useState<{
    show: boolean;
    user: User | null;
    newHandle: string;
  }>({
    show: false,
    user: null,
    newHandle: "",
  });

  const [passwordResetModal, setPasswordResetModal] = useState<{
    show: boolean;
    user: User | null;
    newPassword: string;
  }>({
    show: false,
    user: null,
    newPassword: "",
  });

  const resetState = () => {
    setUsers([]);
    setError(null);
    setLoading(false);
  };

  const fetchUserProfile = async (did: string): Promise<User | null> => {
    try {
      const baseUrl = await Settings.getServiceUrl();
      if (!baseUrl) throw new Error("Service URL not configured");

      const headers = {
        Authorization: await getAuthHeader(),
        "Content-Type": "application/json",
      };

      // First get the account info which includes email
      const accountResponse = await fetch(
        `${baseUrl}/xrpc/com.atproto.admin.getAccountInfo?did=${did}`,
        { headers }
      );

      if (!accountResponse.ok) {
        throw new Error(`Failed to fetch account info: ${accountResponse.status}`);
      }

      const accountData = await accountResponse.json();

      // Then get the profile data as before
      const profileResponse = await fetch(
        `${baseUrl}/xrpc/com.atproto.repo.getRecord?collection=app.bsky.actor.profile&repo=${did}&rkey=self`,
        { headers }
      );

      if (!profileResponse.ok) {
        throw new Error(`Failed to fetch profile: ${profileResponse.status}`);
      }

      const profileData = await profileResponse.json();

      return {
        did,
        email: accountData.email, // Add email from account info
        handle: profileData.value.handle,
        displayName: profileData.value.displayName,
        description: profileData.value.description,
        avatar: profileData.value.avatar,
        createdAt: profileData.value.createdAt,
      };
    } catch (error) {
      console.error(`Error fetching profile for ${did}:`, error);
      return null;
    }
  };


  const deleteUser = async (did: string) => {
    try {
      setLoading(true);
      setError(null);

      const baseUrl = await Settings.getServiceUrl();
      if (!baseUrl) throw new Error("Service URL not configured");

      const headers = {
        Authorization: await getAuthHeader(),
        "Content-Type": "application/json",
      };

      const response = await fetch(
        `${baseUrl}/xrpc/com.atproto.admin.deleteAccount`,
        {
          method: "POST",
          headers,
          body: JSON.stringify({ did }),
        },
      );

      if (!response.ok) {
        const errorText = await response.text();
        console.error("Error response:", errorText);
        throw new Error(
          `Failed to delete user: ${response.status} ${response.statusText}`,
        );
      }

      await fetchUsers();
    } catch (err) {
      console.error("Delete error:", err);
      setError(err instanceof Error ? err.message : "Failed to delete user");
    } finally {
      setLoading(false);
    }
  };
  const fetchUsers = async () => {
    try {
      setLoading(true);
      setError(null);

      const baseUrl = await Settings.getServiceUrl();
      if (!baseUrl) throw new Error("Service URL not configured");

      const headers = {
        Authorization: await getAuthHeader(),
        "Content-Type": "application/json",
      };

      const params = new URLSearchParams({
        limit: "100",
      });

      const response = await fetch(
        `${baseUrl}/xrpc/com.atproto.sync.listRepos?${params}`,
        { headers },
      );

      if (!response.ok) {
        const errorText = await response.text();
        console.error("Error response:", errorText);
        resetState();
        throw new Error(
          `Failed to fetch users: ${response.status} ${response.statusText}`,
        );
      }

      const data: RepoResponse = await response.json();
      const userProfiles = await Promise.all(
        data.repos.map((repo) => fetchUserProfile(repo.did)),
      );

      setUsers(
        userProfiles
          .filter((user): user is User => user !== null)
          .sort(
            (a, b) =>
              new Date(b.createdAt).getTime() - new Date(a.createdAt).getTime(),
          ),
      );
    } catch (err) {
      console.error("Fetch error:", err);
      resetState();
      setError(err instanceof Error ? err.message : "Failed to fetch users");
    } finally {
      setLoading(false);
    }
  };

  const updateUserHandle = async (did: string, newHandle: string) => {
    try {
      setLoading(true);
      setError(null);

      const baseUrl = await Settings.getServiceUrl();
      if (!baseUrl) throw new Error("Service URL not configured");

      const headers = {
        Authorization: await getAuthHeader(),
        "Content-Type": "application/json",
      };

      const response = await fetch(
        `${baseUrl}/xrpc/com.atproto.admin.updateAccountHandle`,
        {
          method: "POST",
          headers,
          body: JSON.stringify({ 
            did,
            handle: newHandle,
          }),
        }
      );

      if (!response.ok) {
        const errorText = await response.text();
        console.error("Error response:", errorText);
        throw new Error(
          `Failed to update handle: ${response.status} ${response.statusText}`
        );
      }

      await fetchUsers(); // Refresh the user list after successful update
    } catch (err) {
      console.error("Update handle error:", err);
      setError(err instanceof Error ? err.message : "Failed to update handle");
    } finally {
      setLoading(false);
    }
  };

  const requestPasswordResetEmail = async (user: User) => {
    try {
      setLoading(true);
      setError(null);

      const baseUrl = await Settings.getServiceUrl();
      if (!baseUrl) throw new Error("Service URL not configured");

      const headers = {
        Authorization: await getAuthHeader(),
        "Content-Type": "application/json",
      };

      const response = await fetch(
        `${baseUrl}/xrpc/com.atproto.server.requestPasswordReset`,
        {
          method: "POST",
          headers,
          body: JSON.stringify({
            email: user.email,
          }),
        }
      );

      if (response.ok) {
        setSuccessMessage(`Password reset email sent to ${user.email}`);
        setTimeout(() => setSuccessMessage(null), 3000);
      }

      if (!response.ok) {
        const errorText = await response.text();
        console.error("Error response:", errorText);
        throw new Error(
          `Failed to send reset email: ${response.status} ${response.statusText}`
        );
      }
    } catch (err) {
      console.error("Password reset email error:", err);
      setError(
        err instanceof Error ? err.message : "Failed to send password reset email"
      );
    } finally {
      setLoading(false);
    }
  };


  const resetUserPassword = async (did: string, newPassword: string) => {
    try {
      setLoading(true);
      setError(null);

      const baseUrl = await Settings.getServiceUrl();
      if (!baseUrl) throw new Error("Service URL not configured");

      const headers = {
        Authorization: await getAuthHeader(),
        "Content-Type": "application/json",
      };

      const response = await fetch(
        `${baseUrl}/xrpc/com.atproto.admin.updateAccountPassword`,
        {
          method: "POST",
          headers,
          body: JSON.stringify({
            did,
            password: newPassword,
          }),
        }
      );

      if (!response.ok) {
        const errorText = await response.text();
        console.error("Error response:", errorText);
        throw new Error(
          `Failed to reset password: ${response.status} ${response.statusText}`
        );
      }
    } catch (err) {
      console.error("Password reset error:", err);
      setError(err instanceof Error ? err.message : "Failed to reset password");
    } finally {
      setLoading(false);
    }
  };


  useEffect(() => {
    async function checkSettings() {
      try {
        const serviceUrl = await Settings.getServiceUrl();
        const adminPassword = await Settings.getAdminPassword();
        setHasValidSettings(Boolean(serviceUrl && adminPassword));
      } catch (error) {
        setHasValidSettings(false);
      } finally {
        setSettingsLoaded(true);
      }
    }

    if (typeof window !== "undefined") {
      checkSettings();
    }
  }, [lastUpdate]);

  useEffect(() => {
    if (typeof window !== "undefined" && hasValidSettings) {
      fetchUsers();
    }
  }, [lastUpdate, hasValidSettings]);

  useEffect(() => {
    return () => {
      resetState();
    };
  }, []);

  if (typeof window === "undefined") {
    return <div></div>;
  }

  if (!settingsLoaded) {
    return (
      <div className="card bg-base-100 shadow-xl">
        <div className="card-body">
          <div className="flex justify-center p-8">
            <span className="loading loading-spinner loading-lg"></span>
          </div>
        </div>
      </div>
    );
  }

  if (!hasValidSettings) {
    return (
      <div className="card bg-base-100 shadow-xl">
        <div className="card-body">
          <div className="alert alert-warning">
            <svg
              xmlns="http://www.w3.org/2000/svg"
              className="stroke-current shrink-0 h-6 w-6"
              fill="none"
              viewBox="0 0 24 24"
            >
              <path
                strokeLinecap="round"
                strokeLinejoin="round"
                strokeWidth="2"
                d="M12 9v2m0 4h.01m-6.938 4h13.856c1.54 0 2.502-1.667 1.732-3L13.732 4c-.77-1.333-2.694-1.333-3.464 0L3.34 16c-.77 1.333.192 3 1.732 3z"
              />
            </svg>
            <div>
              <h3 className="font-bold">Settings Required</h3>
              <div className="text-xs">
                Please configure the service URL and admin password in settings.
              </div>
            </div>
          </div>
        </div>
      </div>
    );
  }

  return (
    <div className="card bg-base-100 shadow-xl">
      <div className="card-body">
        <div className="flex justify-between items-center mb-4">
          <h1 className="card-title text-2xl">Users</h1>
          <button
            onClick={() => fetchUsers()}
            disabled={loading}
            className="btn btn-primary"
          >
            {loading ? (
              <span className="loading loading-spinner loading-sm"></span>
            ) : (
              "Refresh Users"
            )}
          </button>
        </div>

        {/* Password Reset Modal */}
        {passwordResetModal.show && passwordResetModal.user && (
          <dialog className="modal modal-open">
            <div className="modal-box">
              <h3 className="font-bold text-lg">Reset Password</h3>
              <p className="py-4">
                Reset password for{" "}
                <span className="font-semibold">
                  {passwordResetModal.user.displayName || "Anonymous"}
                </span>
              </p>
              <div className="form-control">
                <label className="label">
                  <span className="label-text">New Password</span>
                </label>
                <input
                  type="password"
                  placeholder="New password"
                  className="input input-bordered"
                  value={passwordResetModal.newPassword}
                  onChange={(e) =>
                    setPasswordResetModal({
                      ...passwordResetModal,
                      newPassword: e.target.value,
                    })
                  }
                />
              </div>
              <div className="modal-action">
                <button
                  className="btn"
                  onClick={() =>
                    setPasswordResetModal({ show: false, user: null, newPassword: "" })
                  }
                >
                  Cancel
                </button>
                <button
                  className="btn btn-warning"
                  onClick={async () => {
                    if (passwordResetModal.user && passwordResetModal.newPassword) {
                      await resetUserPassword(
                        passwordResetModal.user.did,
                        passwordResetModal.newPassword
                      );
                      setPasswordResetModal({ show: false, user: null, newPassword: "" });
                    }
                  }}
                  disabled={!passwordResetModal.newPassword}
                >
                  Reset Password
                </button>
              </div>
            </div>
            <form method="dialog" className="modal-backdrop">
              <button
                onClick={() =>
                  setPasswordResetModal({ show: false, user: null, newPassword: "" })
                }
              >
                close
              </button>
            </form>
          </dialog>
        )}

        {/* Edit Handle Modal */}
        {editModal.show && editModal.user && (
          <dialog className="modal modal-open">
            <div className="modal-box">
              <h3 className="font-bold text-lg">Edit Handle</h3>
              <p className="py-4">
                Update handle for{" "}
                <span className="font-semibold">
                  {editModal.user.displayName || "Anonymous"}
                </span>.
                <br />
                <br />
                <span className="font-bold">**Note that you will need a "_atproto" TXT record with the corresponding "did=SOME_DID_NUMBER" before changing your handle to a subdomain of your PDS domain.</span>
              </p>
              <div className="form-control">
                <label className="label">
                  <span className="label-text">New Handle</span>
                </label>
                <input
                  type="text"
                  placeholder="new.handle.pds.domain"
                  className="input input-bordered"
                  value={editModal.newHandle}
                  onChange={(e) =>
                    setEditModal({ ...editModal, newHandle: e.target.value })
                  }
                />
              </div>
              <div className="modal-action">
                <button
                  className="btn"
                  onClick={() =>
                    setEditModal({ show: false, user: null, newHandle: "" })
                  }
                >
                  Cancel
                </button>
                <button
                  className="btn btn-primary"
                  onClick={async () => {
                    if (editModal.user && editModal.newHandle) {
                      await updateUserHandle(editModal.user.did, editModal.newHandle);
                      setEditModal({ show: false, user: null, newHandle: "" });
                    }
                  }}
                  disabled={!editModal.newHandle}
                >
                  Update Handle
                </button>
              </div>
            </div>
            <form method="dialog" className="modal-backdrop">
              <button
                onClick={() =>
                  setEditModal({ show: false, user: null, newHandle: "" })
                }
              >
                close
              </button>
            </form>
          </dialog>
        )}

        {/* Confirmation Modal */}
        {deleteConfirmation.show && deleteConfirmation.user && (
          <dialog className="modal modal-open">
            <div className="modal-box">
              <h3 className="font-bold text-lg text-error">Confirm Delete</h3>
              <p className="py-4">
                Are you sure you want to delete the account for{" "}
                <span className="font-semibold">
                  {deleteConfirmation.user.displayName || "Anonymous"}
                </span>
                ?
              </p>
              <p className="text-sm text-base-content/70 mb-4">
                DID: {deleteConfirmation.user.did}
              </p>
              <div className="modal-action">
                <button
                  className="btn"
                  onClick={() =>
                    setDeleteConfirmation({ show: false, user: null })
                  }
                >
                  Cancel
                </button>
                <button
                  className="btn btn-error"
                  onClick={async () => {
                    if (deleteConfirmation.user) {
                      await deleteUser(deleteConfirmation.user.did);
                      setDeleteConfirmation({ show: false, user: null });
                    }
                  }}
                >
                  Delete Account
                </button>
              </div>
            </div>
            <form method="dialog" className="modal-backdrop">
              <button
                onClick={() =>
                  setDeleteConfirmation({ show: false, user: null })
                }
              >
                close
              </button>
            </form>
          </dialog>
        )}

        {error && (
          <div className="alert alert-error">
            <svg
              xmlns="http://www.w3.org/2000/svg"
              className="stroke-current shrink-0 h-6 w-6"
              fill="none"
              viewBox="0 0 24 24"
            >
              <path
                strokeLinecap="round"
                strokeLinejoin="round"
                strokeWidth="2"
                d="M10 14l2-2m0 0l2-2m-2 2l-2-2m2 2l2 2m7-2a9 9 0 11-18 0 9 9 0 0118 0z"
              />
            </svg>
            <div>
              <h3 className="font-bold">Error!</h3>
              <div className="text-xs">{error}</div>
            </div>
            <button onClick={() => fetchUsers()} className="btn btn-sm">
              Retry
            </button>
          </div>
        )}

        {successMessage && (
          <div className="alert alert-success">
            <svg
              xmlns="http://www.w3.org/2000/svg"
              className="stroke-current shrink-0 h-6 w-6"
              fill="none"
              viewBox="0 0 24 24"
            >
              <path
                strokeLinecap="round"
                strokeLinejoin="round"
                strokeWidth="2"
                d="M9 12l2 2 4-4m6 2a9 9 0 11-18 0 9 9 0 0118 0z"
              />
            </svg>
            <span>{successMessage}</span>
          </div>
        )}

        {loading ? (
          <div className="flex justify-center p-8">
            <span className="loading loading-spinner loading-lg"></span>
          </div>
        ) : (
          <div className="space-y-4">
            {users.map((user) => (
              <div
                key={user.did}
                className="card bg-base-200 hover:shadow-md transition-shadow"
              >
                <div className="card-body">
                  <div className="flex items-start gap-4">
                    <UserAvatar user={user} />
                    <div className="flex-1">
                      <h2 className="text-lg font-semibold">
                        {user.displayName || "Anonymous"}
                      </h2>
                      <div className="font-mono text-sm opacity-70 mt-1">
                        {user.handle}
                      </div>
                      <div className="font-mono text-sm opacity-70 mt-1">
                        {user.did}
                      </div>
                      {user.description && (
                        <p className="text-sm mt-2 opacity-70 line-clamp-2">
                          {user.description}
                        </p>
                      )}
                    </div>

                    <div className="flex flex-col gap-2">
                      <button
                        onClick={() => setEditModal({ 
                          show: true, 
                          user, 
                          newHandle: user.handle || "" 
                        })}
                        className="btn btn-sm btn-primary"
                        disabled={loading}
                      >
                        Edit Handle
                      </button>
                      <button
                        onClick={() => setPasswordResetModal({
                          show: true,
                          user,
                          newPassword: "",
                        })}
                        className="btn btn-sm btn-warning"
                        disabled={loading}
                      >
                        Reset Password
                      </button>
                      <button
                        onClick={() => {
                          if (user.email) {
                            requestPasswordResetEmail(user);
                          } else {
                            setError("No email address available for this user");
                          }
                        }}
                        className="btn btn-sm btn-info"
                        disabled={loading || !user.email}
                        title={user.email ? "Send password reset email" : "No email available"}
                      >
                        Send Reset Email
                      </button>
                      <button
                        onClick={() => setDeleteConfirmation({ show: true, user })}
                        className="btn btn-sm btn-error"
                        disabled={loading}
                      >
                        Delete
                      </button>
                    </div>

                  </div>
                  <div className="flex flex-wrap gap-2 mt-2">
                    <div className="badge badge-ghost">
                      Joined: {new Date(user.createdAt).toLocaleDateString()}
                    </div>
                  </div>
                </div>
              </div>
            ))}
            {!loading && users.length === 0 && (
              <div className="text-center p-8 bg-base-200 rounded-box">
                <div className="text-base-content/70">No users found</div>
              </div>
            )}
          </div>
        )}
      </div>
    </div>
  );
}
