export class Settings {
  private static readonly ENCRYPTION_KEY_NAME =
    "pds-manager-settings-encryption-key";
  private static readonly IV_LENGTH = 12;
  private static readonly DB_NAME = "pds-manager-settings";
  private static readonly STORE_NAME = "settings";

  private static async getOrCreateEncryptionKey(): Promise<CryptoKey> {
    const db = await this.getDB();
    const transaction = db.transaction(this.STORE_NAME, "readwrite");
    const store = transaction.objectStore(this.STORE_NAME);

    return new Promise<CryptoKey>(async (resolve, reject) => {
      try {
        const request = store.get(this.ENCRYPTION_KEY_NAME);

        request.onsuccess = async () => {
          if (request.result) {
            const keyBuffer = request.result;
            const key = await window.crypto.subtle.importKey(
              "raw",
              keyBuffer,
              "AES-GCM",
              true,
              ["encrypt", "decrypt"],
            );
            resolve(key);
          } else {
            // Generate new key if none exists
            const key = await window.crypto.subtle.generateKey(
              {
                name: "AES-GCM",
                length: 256,
              },
              true,
              ["encrypt", "decrypt"],
            );

            // Export and store the key
            const exportedKey = await window.crypto.subtle.exportKey(
              "raw",
              key,
            );

            // Create a new transaction for storing the key
            const newTransaction = db.transaction(this.STORE_NAME, "readwrite");
            const newStore = newTransaction.objectStore(this.STORE_NAME);
            newStore.put(exportedKey, this.ENCRYPTION_KEY_NAME);

            await new Promise<void>((res, rej) => {
              newTransaction.oncomplete = () => res();
              newTransaction.onerror = () => rej(newTransaction.error);
            });

            resolve(key);
          }
        };

        request.onerror = () => reject(request.error);
      } catch (error) {
        reject(error);
      }
    });
  }

  private static async encrypt(data: string): Promise<ArrayBuffer> {
    const key = await this.getOrCreateEncryptionKey();
    const iv = window.crypto.getRandomValues(new Uint8Array(this.IV_LENGTH));
    const encoded = new TextEncoder().encode(data);

    const ciphertext = await window.crypto.subtle.encrypt(
      {
        name: "AES-GCM",
        iv,
      },
      key,
      encoded,
    );

    const resultBuffer = new ArrayBuffer(iv.length + ciphertext.byteLength);
    const resultArray = new Uint8Array(resultBuffer);
    resultArray.set(iv);
    resultArray.set(new Uint8Array(ciphertext), iv.length);

    return resultBuffer;
  }

  private static async decrypt(data: ArrayBuffer): Promise<string> {
    const key = await this.getOrCreateEncryptionKey();
    const iv = new Uint8Array(data, 0, this.IV_LENGTH);
    const ciphertext = new Uint8Array(data, this.IV_LENGTH);

    const decrypted = await window.crypto.subtle.decrypt(
      {
        name: "AES-GCM",
        iv,
      },
      key,
      ciphertext,
    );

    return new TextDecoder().decode(decrypted);
  }

  private static async getDB(): Promise<IDBDatabase> {
    return new Promise((resolve, reject) => {
      const request = indexedDB.open(this.DB_NAME, 1);

      request.onerror = () => reject(request.error);
      request.onsuccess = () => resolve(request.result);

      request.onupgradeneeded = (event) => {
        const db = (event.target as IDBOpenDBRequest).result;
        db.createObjectStore(this.STORE_NAME);
      };
    });
  }

  static async saveSettings(url: string, password: string): Promise<void> {
    if (typeof window === "undefined") return;

    const encryptedUrl = await this.encrypt(url);
    const encryptedPassword = await this.encrypt(password);

    const db = await this.getDB();
    const transaction = db.transaction(this.STORE_NAME, "readwrite");
    const store = transaction.objectStore(this.STORE_NAME);

    store.put(encryptedUrl, "pds_url");
    store.put(encryptedPassword, "pds_admin_password");

    return new Promise((resolve, reject) => {
      transaction.oncomplete = () => resolve();
      transaction.onerror = () => reject(transaction.error);
    });
  }

  static async getServiceUrl(): Promise<string> {
    if (typeof window === "undefined") return "";

    const db = await this.getDB();
    const transaction = db.transaction(this.STORE_NAME, "readonly");
    const store = transaction.objectStore(this.STORE_NAME);

    return new Promise((resolve, reject) => {
      const request = store.get("pds_url");
      request.onsuccess = async () => {
        if (!request.result) {
          resolve("");
          return;
        }
        const decrypted = await this.decrypt(request.result);
        resolve(decrypted);
      };
      request.onerror = () => reject(request.error);
    });
  }

  static async getAdminPassword(): Promise<string> {
    if (typeof window === "undefined") return "";

    const db = await this.getDB();
    const transaction = db.transaction(this.STORE_NAME, "readonly");
    const store = transaction.objectStore(this.STORE_NAME);

    return new Promise((resolve, reject) => {
      const request = store.get("pds_admin_password");
      request.onsuccess = async () => {
        if (!request.result) {
          resolve("");
          return;
        }
        const decrypted = await this.decrypt(request.result);
        resolve(decrypted);
      };
      request.onerror = () => reject(request.error);
    });
  }

  static async clearSettings(): Promise<void> {
    if (typeof window === "undefined") return;

    const db = await this.getDB();
    const transaction = db.transaction(this.STORE_NAME, "readwrite");
    const store = transaction.objectStore(this.STORE_NAME);

    store.delete("pds_url");
    store.delete("pds_admin_password");

    return new Promise((resolve, reject) => {
      transaction.oncomplete = () => resolve();
      transaction.onerror = () => reject(transaction.error);
    });
  }
}
